using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using Vevo.Shared.Domain;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.SystemServices;

namespace Vevo.Shared.DataAccess
{
    // Call DataAccess functions inside the scope of the class to use database transaction.
    // This class imitates the .NET's TransactionScope class. Normal usage pattern is shown below:
    //
    //      using (DataTransactionScope transaction = new DataTransactionScope())
    //      {
    //          DataAccess.ExecuteNonQuery( ... );
    //          ...
    //          transaction.Complete();
    //      }
    //
    // The transaction will be commited once the method Dispose() has been called or
    // when the transaction has gone out of scope.
    //
    // If there are nested DataTransactionScope objects, all nested objects must be Complete()'ed 
    // first, before the transaction can be commited. Otherwise, if one of the inner object is not
    // Complete()'ed, the whole transaction will be rejected.

    public class DataTransactionScope : IDataTransactionScope, IDisposable
    {
        #region Private
        private enum VotingStatus
        {
            None,
            Commit,
            Rollback
        }

        private class DataTransactionItem
        {
            public Guid ID;
            public VotingStatus Status;

            public DataTransactionItem()
            {
                ID = Guid.NewGuid();
                Status = VotingStatus.None;
            }
        }

        private static IPerRequestDataStorage _dataStorage = new HttpContextPerRequestDataStorage();

        private DataTransactionItem _item;
        private bool _disposed = false;

        private static List<DataTransactionItem> OpenedTransactions
        {
            get
            {
                if (!_dataStorage.Items.Contains( "OpenedTransactions" ))
                    _dataStorage.Items["OpenedTransactions"] = new List<DataTransactionItem>();
                return (List<DataTransactionItem>) _dataStorage.Items["OpenedTransactions"];
            }
            set
            {
                _dataStorage.Items["OpenedTransactions"] = value;
            }
        }

        private static DbTransaction Transaction
        {
            get
            {
                if (_dataStorage.Items.Contains( "DbTransaction" ))
                    return (DbTransaction) _dataStorage.Items["DbTransaction"];
                else
                    return null;
            }
            set
            {
                _dataStorage.Items["DbTransaction"] = value;
            }
        }

        private static DbConnection Connection
        {
            get
            {
                if (_dataStorage.Items.Contains( "DbConnection" ))
                    return (DbConnection) _dataStorage.Items["DbConnection"];
                else
                    return null;
            }
            set
            {
                _dataStorage.Items["DbConnection"] = value;
            }

        }

        private void DisposeNoThrow()
        {
            try
            {
                Dispose();
            }
            catch
            {
            }
        }

        private void AddThisTransactionItem()
        {
            _item = new DataTransactionItem();
            OpenedTransactions.Add( _item );
        }

        private void CloseThisTransactionItem()
        {
            if (_item.Status == VotingStatus.None)
                _item.Status = VotingStatus.Rollback;
        }

        private bool IsTransactionProcessed()
        {
            return (Transaction == null);
        }

        private bool IsConnectionOpening()
        {
            if (Transaction != null &&
                Transaction.Connection != null &&
                Transaction.Connection.State == ConnectionState.Open)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private static void EndTransaction()
        {
            DataAccess.EndTransaction();

            Exception currentEx = null;
            if (Transaction != null)
            {
                try
                {
                    Transaction.Dispose();
                }
                catch (Exception ex)
                {
                    currentEx = ex;
                }
                Transaction = null;
            }

            if (Connection != null)
            {
                try
                {
                    Connection.Dispose();
                }
                catch (Exception ex)
                {
                    currentEx = ex;
                }
                Connection = null;
            }

            OpenedTransactions.Clear();

            if (currentEx != null)
                throw currentEx;
        }

        private VotingStatus GetAccumulatedVoting()
        {
            foreach (DataTransactionItem item in OpenedTransactions)
            {
                if (item.Status == VotingStatus.None)
                    return VotingStatus.None;
                else if (item.Status == VotingStatus.Rollback)
                    return VotingStatus.Rollback;
            }

            return VotingStatus.Commit;
        }

        #endregion

        #region Constructors Destructors

        public DataTransactionScope()
        {
            if (Transaction == null)
            {
                OpenedTransactions.Clear();

                Connection = DataAccessHelper.CreateConnection();
                Connection.Open();
                Transaction = Connection.BeginTransaction();
                DataAccess.BeginTransaction( Transaction );
            }

            AddThisTransactionItem();
        }

        ~DataTransactionScope()
        {
            DisposeNoThrow();
        }

        #endregion


        #region Public Methods

        public static void ClearAll()
        {
            EndTransaction();
        }

        public void Dispose()
        {
            if (!_disposed)
            {
                CloseThisTransactionItem();

                if (!IsTransactionProcessed())
                {
                    VotingStatus status = GetAccumulatedVoting();
                    switch (status)
                    {
                        case VotingStatus.None:
                            break;

                        case VotingStatus.Commit:
                            try
                            {
                                if (IsConnectionOpening())
                                    Transaction.Commit();
                            }
                            finally
                            {
                                EndTransaction();
                            }
                            break;

                        case VotingStatus.Rollback:
                            try
                            {
                                if (IsConnectionOpening())
                                    Transaction.Rollback();
                            }
                            finally
                            {
                                EndTransaction();
                            }
                            break;
                    }
                }
            }
            _disposed = true;
        }

        public void Complete()
        {
            _item.Status = VotingStatus.Commit;
        }

        // Set data storage during startup. This storage will keep sensitive 
        // data that should be unique for each HTTP request/response cycle only, 
        // not accessible globally across the HTTP requests.
        public static void SetUpPerRequestDataStorage( IPerRequestDataStorage Storage )
        {
            _dataStorage = Storage;
        }

        #endregion

    }
}
