﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Users;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;

namespace Vevo.Data
{
    public class CustomerRewardPointRepositoryNoCache : ICustomerRewardPointRepository
    {
        #region Private

        private const string AllCustomerRewardPointFields =
           "RewardPointID, CustomerID, Point, Reference, " +
           "ReferenceDate, StoreID, OrderID ";

        private CustomerRewardPoint Create( CustomerRewardPoint customerRewardPoint )
        {
            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO CustomerRewardPoint ( CustomerID, Point, Reference, ReferenceDate, StoreID, OrderID ) " +
                "VALUES ( @CustomerID, @Point, @Reference, @ReferenceDate, @StoreID, @OrderID ) " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( customerRewardPoint.CustomerID ),
                DataAccess.CreateParameterInt32( customerRewardPoint.Point ),
                DataAccess.CreateParameterString( customerRewardPoint.Reference ),
                DataAccess.CreateParameterDateTime( customerRewardPoint.ReferenceDate ),
                DataAccess.CreateParameterString( customerRewardPoint.StoreID ),
                DataAccess.CreateParameterString( customerRewardPoint.OrderID ) );

            customerRewardPoint.RewardPointID = newID;
            return customerRewardPoint;
        }

        private CustomerRewardPoint Update( CustomerRewardPoint customerRewardPoint )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE CustomerRewardPoint " +
                "SET " +
                "   CustomerID = @CustomerID, " +
                "   Point = @Point, " +
                "   Reference = @Reference, " +
                "   ReferenceDate = @ReferenceDate, " +
                "   StoreID = @StoreID, " +
                "   OrderID = @OrderID " +
                "   WHERE RewardPointID = @RewardPointID;",
                DataAccess.CreateParameterString( customerRewardPoint.CustomerID ),
                DataAccess.CreateParameterInt32( customerRewardPoint.Point ),
                DataAccess.CreateParameterString( customerRewardPoint.Reference ),
                DataAccess.CreateParameterDateTime( customerRewardPoint.ReferenceDate ),
                DataAccess.CreateParameterString( customerRewardPoint.StoreID ),
                DataAccess.CreateParameterString( customerRewardPoint.OrderID ),
                DataAccess.CreateParameterString( customerRewardPoint.RewardPointID ) );

            return customerRewardPoint;
        }

        private CustomerRewardPoint CreateOneFromRow( DataRow row )
        {
            CustomerRewardPoint customerRewardPoint = new CustomerRewardPoint();
            customerRewardPoint.RewardPointID = row["RewardPointID"].ToString();
            customerRewardPoint.CustomerID = row["CustomerID"].ToString();
            customerRewardPoint.Point = ConvertUtilities.ToInt32( row["Point"].ToString() );
            customerRewardPoint.Reference = row["Reference"].ToString();
            customerRewardPoint.ReferenceDate = ConvertUtilities.ToDateTime( row["ReferenceDate"].ToString() );
            customerRewardPoint.StoreID = row["StoreID"].ToString();
            customerRewardPoint.OrderID = row["OrderID"].ToString();

            return customerRewardPoint;
        }

        private IList<CustomerRewardPoint> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<CustomerRewardPoint> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<CustomerRewardPoint> list = new List<CustomerRewardPoint>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }

        private DataTable SearchCustomerRewardPointByCustomerIDAndStoreID(
            string customerID,
            string storeID,
            string sortBy,
            int startIndex,
            int endIndex )
        {
            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByText( sortBy ) + ") as rownum, " +
                " * FROM ( " +
                " SELECT " + AllCustomerRewardPointFields +
                " FROM CustomerRewardPoint " +
                " WHERE CustomerID = @CustomerID AND StoreID = @StoreID " +
                ") AS CustomerRewardPoint";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " SELECT * FROM SearchResult WHERE rownum BETWEEN " + (startIndex + 1).ToString() +
                    " AND " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect( extQuery,
                DataAccess.CreateParameterString( customerID ),
                DataAccess.CreateParameterString( storeID ) );

            return table;
        }        

        private int GetCountSearchCustomerRewardPoint(
            string customerID,
            string storeID,
            string sortBy )
        {
            string query = "SELECT COUNT(*) FROM (" +
                " SELECT " + AllCustomerRewardPointFields +
                " FROM CustomerRewardPoint " +
                " WHERE CustomerID = @CustomerID AND StoreID = @StoreID " +
                ") AS Row";

            string countResult = DataAccess.ExecuteScalar( query,
                DataAccess.CreateParameterString( customerID ),
                DataAccess.CreateParameterString( storeID ) );
            return Convert.ToInt32( countResult );
        }

        #endregion

        #region Public Methods

        public CustomerRewardPoint Save( CustomerRewardPoint customerRewardPoint )
        {
            CustomerRewardPoint result;

            if (customerRewardPoint.RewardPointID == "0")
                result = Create( customerRewardPoint );
            else
                result = Update( customerRewardPoint );

            return result;
        }

        public void Delete( string customerRewardPointID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM CustomerRewardPoint " +
                "WHERE RewardPointID = @RewardPointID;",
                DataAccess.CreateParameterString( customerRewardPointID ) );
        }

        public CustomerRewardPoint GetOne( string rewardPointID )
        {
            DataTable table = DataAccess.ExecuteSelect(
            "SELECT " + AllCustomerRewardPointFields +
            "FROM CustomerRewardPoint " +
            "WHERE RewardPointID = @RewardPointID; ",
            DataAccess.CreateParameterString( rewardPointID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return CustomerRewardPoint.Null;

        }

        public CustomerRewardPoint GetOneByOrderID( string orderID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllCustomerRewardPointFields +
                "FROM CustomerRewardPoint " +
                "WHERE OrderID = @OrderID; ",
                DataAccess.CreateParameterString( orderID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return CustomerRewardPoint.Null;
        }

        public IList<CustomerRewardPoint> GetByCustomerIDAndStoreID( string customerID, string storeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT " + AllCustomerRewardPointFields +
               "FROM CustomerRewardPoint " +
               "WHERE CustomerID = @CustomerID AND StoreID = @StoreID; ",
               DataAccess.CreateParameterString( customerID ),
               DataAccess.CreateParameterString( storeID ) );

            return ConvertTableToList( table );
        }

        public decimal SumCustomerIDAndStoreID( string customerID, string storeID )
        {
            string totalPoint = DataAccess.ExecuteScalar(
               "SELECT SUM( Point )" +
               "FROM CustomerRewardPoint " +
               "WHERE CustomerID = @CustomerID AND StoreID = @StoreID; ",
               DataAccess.CreateParameterString( customerID ),
               DataAccess.CreateParameterString( storeID ) );

            return ConvertUtilities.ToDecimal( totalPoint );
        }

        public IList<CustomerRewardPoint> SearchCustomerRewardPointByCustomerIDAndStoreID(
            string customerID,
            string storeID,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            DataTable table = SearchCustomerRewardPointByCustomerIDAndStoreID( customerID, storeID, sortBy, startIndex, endIndex );
            howManyItems = GetCountSearchCustomerRewardPoint( customerID, storeID, sortBy );

            return ConvertTableToList( table );
        }

        public IList<CustomerRewardPoint> SearchCustomerRewardPointByCustomerIDAndStoreID(
            string customerID,
            string storeID,
            string sortBy )
        {
            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByText( sortBy ) + ") as rownum, " +
                " * FROM ( " +
                " SELECT " + AllCustomerRewardPointFields +
                " FROM CustomerRewardPoint " +
                " WHERE CustomerID = @CustomerID AND StoreID = @StoreID " +
                ") AS CustomerRewardPoint";

            DataTable table = DataAccess.ExecuteSelect( query,
                DataAccess.CreateParameterString( customerID ),
                DataAccess.CreateParameterString( storeID ) );

            return ConvertTableToList( table );
        }

        #endregion
    }
}
