using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Shipping;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ShippingWeightRateRepositoryNoCache : IShippingWeightRateRepository
    {
        #region Private

        private const string AllFields = "ShippingWeightRateID, ShippingID, ToWeight, ShippingWeightRate ";

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private ShippingWeightRate Create( ShippingWeightRate shippingWeightRate )
        {
            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO ShippingWeightRate (ShippingID, ToWeight, ShippingWeightRate) " +
                "VALUES (@ShippingID, @ToWeight, @ShippingWeightRate);" +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( shippingWeightRate.ShippingID ),
                DataAccess.CreateParameterDouble( shippingWeightRate.ToWeight ),
                DataAccess.CreateParameterDecimalMoney( shippingWeightRate.WeightRate )
            );
            shippingWeightRate.ShippingWeightRateID = newID;
            return shippingWeightRate;
        }

        private ShippingWeightRate Update( ShippingWeightRate shippingWeightRate )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE ShippingWeightRate " +
                "SET ToWeight = @ToWeight, " +
                "ShippingWeightRate = @ShippingWeightRate " +
                "WHERE ShippingWeightRateID = @ShippingWeightRateID;",
                DataAccess.CreateParameterDouble( shippingWeightRate.ToWeight ),
                DataAccess.CreateParameterDecimalMoney( shippingWeightRate.WeightRate ),
                DataAccess.CreateParameterString( shippingWeightRate.ShippingWeightRateID )
            );
            return shippingWeightRate;
        }

        private ShippingWeightRate CreateOneFromRow( DataRow row )
        {
            ShippingWeightRate shippingWeightRate = new ShippingWeightRate();

            shippingWeightRate.ShippingWeightRateID = row["ShippingWeightRateID"].ToString();
            shippingWeightRate.ShippingID = row["ShippingID"].ToString();
            shippingWeightRate.ToWeight = ConvertUtilities.ToDouble( row["ToWeight"] );
            shippingWeightRate.WeightRate = ConvertUtilities.ToDecimal( row["ShippingWeightRate"] );

            return shippingWeightRate;
        }

        private IList<ShippingWeightRate> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<ShippingWeightRate> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<ShippingWeightRate> list = new List<ShippingWeightRate>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        #endregion


        #region Public Methods

        public ShippingWeightRate Save( ShippingWeightRate shippingWeightRate )
        {
            ShippingWeightRate result;
            if (shippingWeightRate.ShippingWeightRateID == "0")
                result = Create( shippingWeightRate );
            else
                result = Update( shippingWeightRate );
            return result;
        }

        public ShippingWeightRate GetOne( string shippingWeightRateID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                " FROM ShippingWeightRate " +
                "WHERE ShippingWeightRateID = @ShippingWeightRateID;",
                DataAccess.CreateParameterString( shippingWeightRateID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return null;
        }

        public void Delete( string shippingWeightRateID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ShippingWeightRate " +
                "WHERE ShippingWeightRateID = @ShippingWeightRateID;",
                DataAccess.CreateParameterString( shippingWeightRateID )
            );
        }

        public void DeleteByShippingID( string shippingID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ShippingWeightRate " +
                "WHERE ShippingID = @ShippingID;",
                DataAccess.CreateParameterString( shippingID )
            );
        }

        public IList<ShippingWeightRate> GetAllByShippingID( string shippingID, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM ShippingWeightRate " +
                "WHERE ShippingID = @ShippingID " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( shippingID )
                );
            DataColumn[] keys = new DataColumn[1] { table.Columns["ShippingWeightRateID"] };
            table.PrimaryKey = keys;
            return ConvertTableToList( table );
        }

        public IList<ShippingWeightRate> SearchShippingWeightRateItem(
            string shippingID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, searchFilter.FieldName, parameterList );
            parameterList.Add( DataAccess.CreateParameterString( shippingID ) );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                " FROM ShippingWeightRate " +
                commandSearch +
                "AND ShippingID = @ShippingID " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["ShippingWeightRateID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( full, startIndex, endIndex );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM  ShippingWeightRate; "
                );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion
    }
}
