﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Domain.Base;
using Vevo.Shared.DataAccess;
using Vevo.Domain;
using System.Collections;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class SpecificationGroupRepositoryNoCache : ISpecificationGroupRepository
    {
        #region Private

        private const string AllOptionGroupLocaleFields = " SpecificationGroupID , CultureID, DisplayName, Description   ";

        private SpecificationGroupLocale CreateOneLocaleFromRow( DataRow row )
        {
            SpecificationGroupLocale locale = new SpecificationGroupLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.DisplayName = row["DisplayName"].ToString();
            locale.Description = row["Description"].ToString();
            return locale;
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private SpecificationGroup Create( SpecificationGroup specificationGroup )
        {
            string specificationGroupID = DataAccess.ExecuteScalar(
                "INSERT INTO SpecificationGroup ( Name ) " +
                "VALUES ( @Name );" +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( specificationGroup.Name ) );

            specificationGroup.SpecificationGroupID = specificationGroupID;

            InsertLocales( specificationGroup );

            return specificationGroup;
        }

        private SpecificationGroup Update( SpecificationGroup specificationGroup )
        {
            int update = DataAccess.ExecuteNonQuery(
                "UPDATE SpecificationGroup " +
                "SET Name = @Name " +
                "WHERE SpecificationGroupID = @SpecificationGroupID; ",
                DataAccess.CreateParameterString( specificationGroup.Name ),
                DataAccess.CreateParameterString( specificationGroup.SpecificationGroupID ) );

            DeleteLocales( specificationGroup.SpecificationGroupID );
            InsertLocales( specificationGroup );

            return specificationGroup;
        }

        private void DeleteLocales( string specificationGroupID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM SpecificationGroupLocale " +
                "WHERE SpecificationGroupID  = @SpecificationGroupID ; ",
                DataAccess.CreateParameterString( specificationGroupID ) );
        }

        private void InsertLocales( SpecificationGroup specificationGroup )
        {
            foreach (SpecificationGroupLocale locale in specificationGroup.Locales)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO SpecificationGroupLocale (SpecificationGroupID , CultureID, DisplayName , Description  ) " +
                    "VALUES (@OptionGroupID, @CultureID, @DisplayName, @Description );",
                    DataAccess.CreateParameterString( specificationGroup.SpecificationGroupID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.DisplayName ),
                    DataAccess.CreateParameterString( locale.Description ) );
            }
        }
        private SpecificationGroup CreateOneFromRow( Culture culture, DataRow row )
        {
            SpecificationGroup specificationGroup = new SpecificationGroup( culture );
            specificationGroup.SpecificationGroupID = row["SpecificationGroupID"].ToString();
            specificationGroup.Name = row["Name"].ToString();
            specificationGroup.DisplayName = row["DisplayName"].ToString();
            specificationGroup.Description = row["Description"].ToString();

            return specificationGroup;
        }

        private void CopySpecificationGroupLocales( SpecificationGroup specificationGroup, IList<ILocale> locales )
        {
            foreach (SpecificationGroupLocale locale in locales)
            {
                specificationGroup.Locales.Add( locale );
            }
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "specificationgroupid" ))
                return "ORDER BY SpecificationGroup." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private IList<SpecificationGroup> ConvertTableToList( Culture culture, DataTable table, int startIndex, int endIndex )
        {
            IList<SpecificationGroup> list = new List<SpecificationGroup>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( culture, table.Rows[i] ) );

            return list;
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "specificationgroupid":
                    return "SpecificationGroup.SpecificationGroupID";
                default:
                    return fieldName;
            }
        }

        #endregion

        #region Public Methods
        public SpecificationGroup Save( SpecificationGroup specificationGroup )
        {
            SpecificationGroup result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (specificationGroup.SpecificationGroupID == "0")
                    result = Create( specificationGroup );
                else
                    result = Update( specificationGroup );

                transaction.Complete();
            }

            return result;
        }

        public IList<ILocale> GetLocales( string specificationGroupID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllOptionGroupLocaleFields +
                "FROM SpecificationGroupLocale " +
                "WHERE SpecificationGroupID  = @SpecificationGroupID ; ",
                DataAccess.CreateParameterString( specificationGroupID ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT SpecificationGroup.SpecificationGroupID, Name, DisplayName, Description " +
                "FROM SpecificationGroup LEFT JOIN " +
                "    (SELECT * FROM SpecificationGroupLocale) AS L " +
                "    ON SpecificationGroup.SpecificationGroupID = L.SpecificationGroupID WHERE 1 = 0 " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public void Delete( string specificationGroupID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                "DELETE FROM SpecificationGroup WHERE SpecificationGroupID = @SpecificationGroupID;",
                DataAccess.CreateParameterString( specificationGroupID ) );
                DeleteLocales( specificationGroupID );
                transaction.Complete();
            }
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM SpecificationGroupLocale " +
                "WHERE CultureID = @CultureID; ",
                DataAccess.CreateParameterString( cultureID )
                );
        }

        public SpecificationGroup GetOne( Culture culture, string specificationGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT SpecificationGroup.SpecificationGroupID, Name, DisplayName, Description  " +
               "FROM SpecificationGroup LEFT JOIN " +
               "    (SELECT * FROM SpecificationGroupLocale WHERE CultureID = @CultureID) AS L " +
               "    ON SpecificationGroup.SpecificationGroupID = L.SpecificationGroupID " +
               "WHERE SpecificationGroup.SpecificationGroupID = @SpecificationGroupID;",
               DataAccess.CreateParameterString( culture.CultureID ),
               DataAccess.CreateParameterString( specificationGroupID ) );

            if (table.Rows.Count > 0)
            {
                SpecificationGroup specificationGroup = CreateOneFromRow( culture, table.Rows[0] );
                CopySpecificationGroupLocales( specificationGroup, GetLocales( specificationGroupID ) );

                return specificationGroup;
            }
            else
                return SpecificationGroup.Null;
        }

        public IList<SpecificationGroup> GetAll( Culture culture )
        {
            return GetAll( culture, String.Empty );
        }

        public IList<SpecificationGroup> GetAll( Culture culture, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT SpecificationGroup.SpecificationGroupID, Name, DisplayName, Description   " +
                "FROM SpecificationGroup LEFT JOIN " +
                "    (SELECT * FROM SpecificationGroupLocale WHERE CultureID = @CultureID) AS L " +
                "    ON SpecificationGroup.SpecificationGroupID = L.SpecificationGroupID " +
                "WHERE SpecificationGroup.SpecificationGroupID in (SELECT DISTINCT SpecificationGroupID FROM SpecificationItem) " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );

            return ConvertTableToList( culture, table, 0, table.Rows.Count );
        }

        public IList<SpecificationGroup> SearchSpecificationGroup(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );

            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string optionFieldName = ConvertFieldName( searchFilter.FieldName );
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, optionFieldName, parameterList );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT SpecificationGroup.SpecificationGroupID, Name, DisplayName, Description   " +
                "FROM SpecificationGroup LEFT JOIN " +
                "    (SELECT * FROM SpecificationGroupLocale WHERE CultureID = @CultureID) AS L " +
                "    ON SpecificationGroup.SpecificationGroupID = L.SpecificationGroupID " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["SpecificationGroup.SpecificationGroupID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( culture, full, startIndex, endIndex );
        }

        #endregion
    }
}
