using System;
using System.Data;
using System.Configuration;
using System.Collections.Generic;
using Vevo.Domain.Users;
using Vevo.Domain.Configurations;
using Vevo.Domain.Orders;
using com.paypal.sdk.services;
using com.paypal.sdk.util;
using Vevo.Shared.Domain.SystemServices;

namespace Vevo.Domain.Payments
{
    public class ProcessRecurringPaymentService : ProcessPaymentService
    {
        #region Private

        private bool ProcessRecurringPayment(
             CartItemGroup cartItemGroup,
             CheckoutDetails checkout,
             Culture culture,
             Currency currency,
             Customer customer,
             string storefrontUrl,
             RecurringPaymentResult recurringPaymentResult,
             OrderAmountPackage orderAmountPackage,
             string ipAddress,
             bool isLastProduct,
             out string errorMessage,
             out string paymentLog )
        {
            ICartItem item = cartItemGroup.CartItems[0];

            PaymentAppGateway paymentAppGateway = new PaymentAppGateway( checkout );

            string postData = paymentAppGateway.CreateOnWebsiteRecurringPaymentXml(
                                culture,
                                currency,
                                orderAmountPackage,
                                item,
                                ipAddress,
                                storefrontUrl,
                                isLastProduct );

            PaymentAppResult paymentResult = paymentAppGateway.PostCommand( this.HttpService, postData, storefrontUrl );
            paymentLog = paymentResult.PaymentLog;
            errorMessage = paymentResult.ErrorMessage;

            if (paymentResult.Status == PaymentAppResult.PaymentStatus.OK)
            {
                recurringPaymentResult.AddRecurringID( item.CartItemID, paymentResult.ReferenceID );
                return true;
            }
            else
                return false;
        }

        private bool IsLastProduct( Cart cart, int recurringProductCount )
        {
            if (!cart.ContainsNonRecurringProduct() &&
                recurringProductCount == cart.NumberOfRecurringProduct())
            {
                return true;
            }
            return false;
        }

        private bool ProcessAllRecurringItems(
            Currency currency,
            Culture culture,
            CheckoutDetails checkout,
            Cart cart,
            Customer customer,
            string storefrontUrl,
            string ipAddress,
            out PaymentAppResult paymentResult,
            out RecurringPaymentResult recurringPaymentResult,
            out string errorMessage,
            out string paymentLog )
        {
            int loopCount = 0;
            int recurringProductCount = 0;

            paymentResult = new PaymentAppResult();
            recurringPaymentResult = new RecurringPaymentResult();

            errorMessage = String.Empty;
            paymentLog = String.Empty;

            foreach (CartItemGroup cartItemGroup in cart.SeparateCartItemGroups())
            {
                if (cartItemGroup.IsRecurring)
                {
                    recurringProductCount++;

                    bool isLastProduct = IsLastProduct( cart, recurringProductCount );

                    OrderCalculator orderCalculator = new OrderCalculator();
                    OrderAmountPackage orderAmountPackage = orderCalculator.CalculatePackage( checkout, cartItemGroup, customer );

                    string recurringMessage, recurringLog;

                    bool result = ProcessRecurringPayment(
                            cartItemGroup,
                            checkout,
                            culture,
                            currency,
                            customer,
                            storefrontUrl,
                            recurringPaymentResult,
                            orderAmountPackage,
                            ipAddress,
                            isLastProduct,
                            out recurringMessage,
                            out recurringLog );
                    errorMessage += recurringMessage;
                    paymentLog += recurringLog;

                    if (!result)
                    {
                        if (loopCount == 0)
                        {
                            paymentResult = new PaymentAppResult(
                                PaymentAppResult.PaymentStatus.Error,
                                String.Empty,
                                errorMessage,
                                paymentLog,
                                String.Empty );

                            return false;
                        }
                        else
                        {
                            ICartItem item = cartItemGroup.CartItems[0];

                            cart.DeleteItem( item.CartItemID );
                        }
                    }
                }
                loopCount++;
            }

            return true;
        }

        #endregion


        #region Constructors

        public ProcessRecurringPaymentService( IHttpService httpService )
            : base( httpService )
        {
        }

        #endregion


        #region Public Methods

        public override bool ProcessPayPalProUSPayment(
            decimal amount,
            Currency currency,
            Culture culture,
            CheckoutDetails checkout,
            Cart cart,
            Customer customer,
            string storefrontUrl,
            string ipAddress,
            out PaymentAppResult paymentResult,
            out RecurringPaymentResult recurringPaymentResult )
        {
            bool processFlag = true;

            PaymentErrors.Instance.Clear();

            paymentResult = new PaymentAppResult();
            recurringPaymentResult = new RecurringPaymentResult();

            string errorMessage = String.Empty;
            string paymentLog = String.Empty;

            bool isRecurringProcessPass = true;

            isRecurringProcessPass = ProcessAllRecurringItems(
                                        currency,
                                        culture,
                                        checkout,
                                        cart,
                                        customer,
                                        storefrontUrl,
                                        ipAddress,
                                        out paymentResult,
                                        out recurringPaymentResult,
                                        out errorMessage,
                                        out paymentLog );

            if (!isRecurringProcessPass)
                return false;

            CartItemGroup nonRecurringCartItemGroup = GetNonRecurringCartItemGroup( cart );

            if (nonRecurringCartItemGroup != null &&
                amount > 0)
            {
                bool paid = ProcessDirectPayment(
                    checkout,
                    nonRecurringCartItemGroup,
                    culture,
                    currency,
                    customer,
                    storefrontUrl,
                    ipAddress,
                    out paymentResult );

                if (!paid)
                {
                    foreach (ICartItem item in nonRecurringCartItemGroup.CartItems)
                    {
                        cart.DeleteItem( item.CartItemID );
                    }
                    PaymentErrors.Instance.AddErrorMessage( "<br/>Regular Product : " + paymentResult.ErrorMessage );
                }
                errorMessage += paymentResult.ErrorMessage;
                paymentLog += paymentResult.PaymentLog;
            }

            return processFlag;
        }

        #endregion

    }
}
