using System;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using Vevo.Domain;
using Vevo.Domain.Security;
using Vevo.Domain.Stores;
using Vevo.Shared.Utilities;
using Vevo.Shared.WebUI;
using Vevo.WebAppLib;

namespace Vevo.WebUI.BaseControls
{
    public class BaseMasterPage : Vevo.WebUI.International.BaseLanguageMasterPage
    {
        #region Private

        private HtmlLink CreateLink( string href )
        {
            HtmlLink link = new HtmlLink();
            link.Href = href;
            link.Attributes.Add( "rel", "stylesheet" );
            link.Attributes.Add( "type", "text/css" );
            return link;
        }

        private Literal CreateLiteral( string text )
        {
            Literal literal = new Literal();
            literal.Text = text;
            return literal;
        }

        private bool IsPageErrorException()
        {
            foreach (string page in _PageErrorException)
            {
                if (String.Compare( GetPageName( Request.Url.AbsolutePath ), page, true ) == 0)
                {
                    return true;
                }
            }

            return false;
        }

        private string[] _PageErrorException = new string[]
        {
            "GenericError.aspx"
         };

        private string GetPageName( string url )
        {
            string[] splitURL = url.Split( '/' );
            int upperBound = splitURL.GetUpperBound( 0 );

            return splitURL[upperBound].ToString();
        }

        private bool IsTransaction
        {
            get
            {
                if (!String.IsNullOrEmpty( Request.QueryString["IsTransaction"] ))
                    return ConvertUtilities.ToBoolean( Request.QueryString["IsTransaction"] );
                else
                    return false;
            }
        }

        private string OrderID
        {
            get
            {
                if (!String.IsNullOrEmpty( Request.QueryString["OrderID"] ))
                    return Request.QueryString["OrderID"];
                else
                    return "0";
            }
        }

        private void SSLVerifyAndRedirect()
        {
            string newUrl;
            SslPolicy sslPolicy = new SslPolicy();

            // Need to get user-friendly url (e.g. CatalogA-list.aspx 
            // instead of Catalog.aspx?CatalogName=CatalogA.)
            if (sslPolicy.RequiresRedirect( UrlPath.GetDisplayedUrl(), out newUrl ))
            {
                Response.Redirect( newUrl );
            }
        }

        private void AddMetaItem( string name, string content )
        {
            WebUtilities.PageAddMeta( Page, name, content );
        }

        private void SetMetaAndTitle()
        {
            string currentPage = Request.Url.AbsolutePath.ToLower();

            if (!currentPage.Contains( "catalog.aspx" ) &&
                !currentPage.Contains( "product.aspx" ) &&
                !currentPage.Contains( "content.aspx" ))
            {
                AddMetaItem( "description", NamedConfig.SiteDescription );
                AddMetaItem( "keywords", NamedConfig.SiteKeyword );
            }
        }

        private void SetCanonicalization()
        {
            //string currentPage = Request.Url.AbsolutePath.ToLower();

            string pageAndQueryUrl = Request.Url.ToString().Substring( UrlPath.StorefrontUrl.Length - 1 );

            if (pageAndQueryUrl.ToLower().Contains( "-details.aspx" ) ||
                pageAndQueryUrl.ToLower().Contains( "-list.aspx" ) ||
                pageAndQueryUrl.ToLower().Contains( "-items.aspx" ) ||
                pageAndQueryUrl.ToLower().Contains( "catalog.aspx" ) ||
                pageAndQueryUrl.ToLower().Contains( "department.aspx" ) ||
                pageAndQueryUrl.ToLower().Contains( "product.aspx" ))
            {
                WebUtilities.PageCannonicalization( Page, pageAndQueryUrl, StoreContext.Culture );
            }
        }

        private void AddIconLink()
        {
            HtmlLink link = new HtmlLink();
            link.Attributes.Add( "rel", "shortcut icon" );
            link.Attributes.Add( "type", "image/vnd.microsoft.icon" );
            link.Href = "~/favicon.ico";
            Page.Header.Controls.Add( link );
        }

        private void RegisterGoogleAnalytics()
        {
            string script;
            if (IsTransaction)
                script = GoogleAnalyticUtilities.RegisterTransactionScript( OrderID );
            else
                script = GoogleAnalyticUtilities.RegisterTrackingScript();
            if (this.MasterPageFile == null)
            {
                ScriptManager.RegisterStartupScript(this, this.GetType(), "GoogleAnalytics", script, false);
            }
        }

        private void ConfigAjaxAsyncPostBackError()
        {
            if (DataAccessContext.Configurations.GetBoolValueNoThrow( "ShowDetailAjaxErrorMessage" ))
            {
                ScriptManager.GetCurrent( Page ).AsyncPostBackError += ScriptManager_AsyncPostBackError;
            }
        }

        private void RegisterClientScriptLibraries()
        {
            ScriptManager.RegisterClientScriptInclude( Page, typeof( Page ), "ForCallOut", "ClientScripts/Utilities.js" );
            ScriptManager.RegisterClientScriptInclude( Page, typeof( Page ), "jquery", "ClientScripts/JQuery/jquery-1.8.1.min.js" );
            ScriptManager.RegisterClientScriptInclude( Page, typeof( Page ), "jquerycycle", "ClientScripts/JQuery/jquery.cycle.all.min.js" );
            ScriptManager.RegisterClientScriptInclude( Page, typeof( Page ), "jquerypop", "ClientScripts/JQuery/jquery.pop.js" );
            ScriptManager.RegisterClientScriptInclude( Page, typeof( Page ), "jqueryslide", "ClientScripts/JQuery/slides.min.jquery.js" );
        }

        private void CheckRedirectToMobile()
        {
            StoreRetriever storeRetreiver = new StoreRetriever();
            Store store = storeRetreiver.GetStore();
            if (DataAccessContext.Configurations.GetBoolValue( "MobileView", store ) && UrlManager.IsMobileClient())
            {
                if (!String.IsNullOrEmpty( Request.QueryString["IsViewMobile"] ))
                {
                    bool isViewMobile = ConvertUtilities.ToBoolean( Request.QueryString["IsViewMobile"] );
                    Session[SystemConst.IsViewInMobileSessionName] = ConvertUtilities.ToString( isViewMobile );
                }
                if ((Session[SystemConst.IsViewInMobileSessionName] == null) ||
                    (ConvertUtilities.ToBoolean( Session[SystemConst.IsViewInMobileSessionName].ToString() ) == true))
                {
                    string returnUrl = string.Empty;
                    string urlPath = Request.Url.AbsoluteUri;

                    if (urlPath.StartsWith( "http://www." ))
                        returnUrl = urlPath.Substring( store.UrlName.Length + 12 );
                    else if (urlPath.StartsWith( "https://www." ))
                        returnUrl = urlPath.Substring( store.UrlName.Length + 13 );
                    else if (urlPath.StartsWith( "http://" ))
                        returnUrl = urlPath.Substring( store.UrlName.Length + 8 );
                    else if (urlPath.StartsWith( "https://" ))
                        returnUrl = urlPath.Substring( store.UrlName.Length + 9 );

                    Response.Redirect( "~/" + UrlManager.MobileFolder + "/" + returnUrl );
                }
            }
        }

        private void RegisterFixCss()
        {
            String browserName = Request.Browser.Browser.ToLower();

            if (browserName == "ie")
            {
                Page.Header.Controls.Add( CreateLiteral( "<!--[if IE 9]>\n" ) );
                Page.Header.Controls.Add( CreateLink( "~/App_Themes/FrontCommon/DefaultFixIE.css" ) );
                Page.Header.Controls.Add( CreateLiteral( "<![endif]-->" ) );
            }
        }

        private void BaseMasterPage_Load( object sender, EventArgs e )
        {
            if (!DataAccessContext.ApplicationSettings.InstallCompleted && !IsPageErrorException())
            {
                Response.Redirect( "Install/PermissionCheck.aspx" );
            }

            DatabaseConnectionService connectionService = new DatabaseConnectionService();
            if (!connectionService.TryConnect())
            {
                throw new Exception( "Database Connection Error.There is an error while attempting to connect to the database. Please verify your connection string in the file." );
            }

            SSLVerifyAndRedirect();

            SetMetaAndTitle();

            AddIconLink();

            RegisterGoogleAnalytics();

            ConfigAjaxAsyncPostBackError();

            RegisterFixCss();

            RegisterClientScriptLibraries();



            if (KeyUtilities.IsMultistoreLicense())
            {
                SetCanonicalization();
            }

            CheckRedirectToMobile();
        }


        #endregion


        #region Constructors

        public BaseMasterPage()
        {
            Load += new EventHandler( BaseMasterPage_Load );
        }

        #endregion


        #region Protected

        protected void ScriptManager_AsyncPostBackError( object sender, AsyncPostBackErrorEventArgs e )
        {
            ScriptManager.GetCurrent( Page ).AsyncPostBackErrorMessage =
                String.Format( "Exception Type: {0}\n\nMessage: {1}\n\nMethod: {2}\n\nStack Trace:\n{3}",
                    e.Exception.GetType().ToString(),
                    e.Exception.Message,
                    e.Exception.TargetSite,
                    e.Exception.StackTrace );
        }

        protected bool IsRestrictedAccessPage()
        {
            string path = Request.Url.AbsolutePath.ToLower();
            bool isAuthPage = true;

            if (path.Contains( "userlogin.aspx" ) || path.Contains( "register.aspx" ) || path.Contains( "forgotpassword.aspx" ) ||
                path.Contains( "resetpassword.aspx" ) || path.Contains( "passwordrecoveryfinished.aspx" ) ||
                path.Contains( "registerfinish.aspx" ) || path.Contains( "affiliatelogin.aspx" ))
            {
                isAuthPage = false;
            }

            return isAuthPage;
        }

        protected string AffiliateCode
        {
            get
            {
                if (!String.IsNullOrEmpty( Request.QueryString["AffiliateCode"] ))
                    return Request.QueryString["AffiliateCode"];
                else
                    return String.Empty;
            }
        }

        #endregion
    }
}
